﻿using System;
using System.Collections.Generic;
using System.Linq;
using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;

namespace FastCash
{
    public static class Log
    {
        public static void Normal(string message)
        {
            Logging.Write("[FastCash] " + message);
        }
    }

    public class ObjectsDetector : HBPlugin
    {
        // List of items to sell
        private readonly IEnumerable<string> _itemsNames = new[] {"Wilted Lilypad", "Snowplum Brandy"};
        // List of different personal vendors in game
        private readonly IEnumerable<Vendor> _vendors = new Vendor[]
        {
            new Mount(61425), //Gnimo - Alliance Mammoth
            new Mount(61447), //Drix - Horde Mammoth
            new Mount(122708), //Slowhand - Alliance/Horde Yak
            new Pet(92395), //Pet Guild Page - Alliance Guild Page
            new Pet(92396), //Pet Guild Page - Horde Guild Page
            new Pet(92397), //Pet Guild Herald - Alliance Guild Herald
            new Pet(92398), //Pet Guild Herald - Horde Guild Herald
            //new Pet(33238, 0, 62609), //Pet Argent Squire - Argent Squire ===> require item Argent Pony Bridle
            //new Pet(33239, 0, 62746) //Argent Gruntling ==> require item Argent Pony Bridle
            new Profession(18232), // Field Repair Bot 74A
            new Profession(34113), // Field Repair Bot 110G
            new Profession(40769), // Scrapbot Construction Kit
            new Profession(49040), // Jeeves 
            new Profession(40768) // MOLL-E 
        };

        // Currently available vendors
        private IEnumerable<Vendor> _availableVendors;
        // All known vendors of this character
        private IEnumerable<Vendor> _knownVendors;
        private bool _needToSell;

        public override string Name
        {
            get { return "FastCash"; }
        }

        public override string Author
        {
            get { return "Deams"; }
        }

        public override Version Version
        {
            get { return new Version(1, 1, 0, 0); }
        }

        public override void OnEnable()
        {
            Log.Normal("Currently " + _vendors.Count() + " vendors in DB, check the forum for more details.");
            // Let's check all the vendors we know
            _knownVendors = _vendors.Where(v => v.IsKnown());
            if (_knownVendors.Any())
            {
                int numberVendors = _knownVendors.Count();
                Log.Normal("Found " + numberVendors + " vendor" + (numberVendors > 1 ? "s" : "") +
                           " known by this character:");
                string list = _knownVendors.Aggregate("", (current, vendor) => current + (vendor.ToString() + ", "));
                Log.Normal(list);
                Lua.Events.AttachEvent("LOOT_CLOSED", NewLoots);
            }
            else Log.Normal("Couldn't find any possible vendors on this charater. FastCash will stay silent.");
        }

        public override void OnDisable()
        {
            _knownVendors = null;
            _availableVendors = null;
            _needToSell = false;
            Lua.Events.DetachEvent("LOOT_CLOSED", NewLoots);
            Log.Normal("Deactivated, bye!");
        }

        public override void Pulse()
        {
            if (StyxWoW.Me.IsActuallyInCombat || StyxWoW.Me.IsDead || StyxWoW.Me.IsGhost
                || StyxWoW.Me.IsCasting || StyxWoW.Me.IsChanneling || !Styx.CommonBot.Mount.CanMount()) return;
            if (!_needToSell || !_knownVendors.Any()) return;

            // Check if any available vendors in the known one
            _availableVendors = _knownVendors.Where(v => v.CanUse()).OrderBy(v => v.Cooldown);
            if (_availableVendors.Any())
            {
                Vendor v = _availableVendors.First();
                v.Summon();
                v.Sell(_itemsNames);
                if (!Utils.IsinBags(_itemsNames)) _needToSell = false;
            }
            else
            {
                Log.Normal("No available mobile sellers for now, will try after next looting phase.");
                _needToSell = false;
            }
        }

        public void NewLoots(object sender, LuaEventArgs args)
        {
            if (!Utils.IsinBags(_itemsNames)) return;
            _needToSell = true;
            Log.Normal("Found Object to Sell.");
        }
    }
}