﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace FastCash
{
    /*****************************************************
     * Class Vendor
     * Added 28/10/2014
     * 
     *****************************************************/

    public abstract class Vendor
    {
        protected readonly int SkillId;
        public int Cooldown;
        public bool Enabled;

        protected Vendor(int cooldown, int skillId)
        {
            Cooldown = cooldown;
            Enabled = false;
            SkillId = skillId;
        }

        public abstract bool CanUse();
        public abstract bool IsKnown();
        public abstract void Summon();

        public async void Sell(IEnumerable<string> itemsNamesList)
        {
            //interact with vendor
            WoWUnit vendor = ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                .Where(u => u != null && u.IsRepairMerchant && u.IsAlive && !u.IsHostile)
                .OrderBy(u => u.Distance).FirstOrDefault();
            if (vendor == null) return;
            // check if the vendor is the one that correspond to the current instance of vendor
            // TO DO
            // But we keep trying and annoucing it in the log if it's not the right one. 
            if (!vendor.WithinInteractRange) return;
            vendor.Interact();
            if (!MerchantFrame.Instance.IsVisible) return;
            Log.Normal("Selling!");
            foreach (string itemName in itemsNamesList)
            {
                string name = itemName;
                foreach (WoWItem item in StyxWoW.Me.BagItems.Where(item => item.Name == name))
                {
                    item.UseContainerItem();
                }
            }
            ObjectManager.Update();
            await Task.Delay(1000);
        }
    }

    /*****************************************************
     * Class Mount : Vendor
     * Added 28/10/2014
     * 
     *****************************************************/

    public class Mount : Vendor
    {
        private readonly WoWSpell _skillSpell;

        public Mount(int skillId)
            : base(0, skillId)
        {
            _skillSpell = Styx.CommonBot.Mount.Mounts.Where(mount => mount.CreatureSpellId == SkillId)
                .Select(mount => mount.CreatureSpell).FirstOrDefault();
        }

        // MUST ADD ENVIRONMENT PARAMETERS HERE! OR SHOULD IT BE IN THE PULSE DIRECTLY? 
        // NAH IT IS SPECIFIC TO EACH VENDOR, MAYBE YOU CANT MOUNT BUT YOU CAN USE JEEVES


        public override bool CanUse()
        {
            return _skillSpell.CanCast;
        }

        public override bool IsKnown()
        {
            return _skillSpell != null;
        }

        public override async void Summon()
        {
            //Invoke mount
            if (StyxWoW.Me.Mounted && !IsCurrentMount(SkillId))
            {
                CommonCoroutines.LandAndDismount("Dismount to cast mount with vendor and auto sell.");
            }
            if (!StyxWoW.Me.Mounted)
            {
                Logging.Write("Mounting");
                _skillSpell.Cast();
                await Task.Delay(2000);
            }
        }

        private static bool IsCurrentMount(int mountId)
        {
            return StyxWoW.Me.Auras.Values.FirstOrDefault(aura => mountId == aura.SpellId) != null;
        }

        public override string ToString()
        {
            if (_skillSpell != null) return _skillSpell.Name;
            return "ID:" + SkillId;
        }
    }


    /*****************************************************
     * Class Pet : Vendor
     * Added 28/10/2014
     * 
     *****************************************************/

    public class Pet : Vendor
    {
        private readonly WoWSpell _skillSpell;

        public Pet(int skillId)
            : base(GetCd(skillId), skillId)
        {
            _skillSpell = WoWSpell.FromId(skillId);
        }

        private static int GetCd(int skillId)
        {
            return (int) WoWSpell.FromId(skillId).BaseCooldown;
        }

        public override bool CanUse()
        {
            return _skillSpell.CanCast;
        }

        public override bool IsKnown()
        {
            return (_skillSpell != null);
        }

        public override async void Summon()
        {
            SpellManager.Cast(SkillId);
            await Task.Delay(500);
        }

        public override string ToString()
        {
            if (_skillSpell != null) return _skillSpell.Name;
            return "ID:" + SkillId;
        }
    }

    /*****************************************************
     * Class Profession : Vendor
     * Added 28/10/2014
     * 
     *****************************************************/

    public class Profession : Vendor
    {
        private int itemId;

        public Profession(int itemID)
            : base(GetCd(itemID), itemID)
        {
        }

        private static int GetCd(int itemId)
        {
            if (Utils.IsinBags(itemId)) return (int) Utils.GetFromBags(itemId).Cooldown;
            return 0;
        }

        public override bool CanUse()
        {
            WoWItem item = Utils.GetFromBags(itemId);
            if (item != null) return item.Usable;
            return false;
        }

        public override bool IsKnown()
        {
            return Utils.IsinBags(itemId);
        }

        public override async void Summon()
        {
            WoWItem item = Utils.GetFromBags(itemId);
            item.Use();
            await Task.Delay(500);
        }

        public override string ToString()
        {
            return "ID:" + SkillId;
        }
    }
}